@@ -29,14 +29,18 @@ def task_ut():
 
 def task_coverage():
     """show coverage for all modules including tests"""
-    return {'actions':["coverage run `which py.test` ",
-                   "coverage report --show-missing %s" % " ".join(pyFiles)],
+    return {'actions':
+                ["coverage run --parallel-mode `which py.test` ",
+                 "coverage combine",
+                 "coverage report --show-missing %s" % " ".join(pyFiles)],
             'verbosity': 2}
 
 def task_coverage_code():
     """show coverage for all modules (exclude tests)"""
-    return {'actions':["coverage run `which py.test` ",
-                  "coverage report --show-missing %s" % " ".join(codeFiles)],
+    return {'actions':
+                ["coverage run --parallel-mode `which py.test` ",
+                 "coverage combine",
+                 "coverage report --show-missing %s" % " ".join(codeFiles)],
             'verbosity': 2}
 
 def task_coverage_module():
@@ -47,8 +51,10 @@ def task_coverage_module():
             continue
         source = "doit/" + test[to_strip:]
         yield {'name': test,
-               'actions':["coverage run `which py.test` %s" % test,
-                   "coverage report --show-missing %s %s" % (source, test)],
+               'actions':
+                   ["coverage run --parallel-mode `which py.test` -v %s" % test,
+                    "coverage combine",
+                    "coverage report --show-missing %s %s" % (source, test)],
                'verbosity': 2}
 
 
