@@ -1,15 +1,12 @@
+"""Implements actions used by doit tasks
+"""
+
 import subprocess, sys
 import StringIO
 import inspect
 from threading import Thread
 
-from doit.exceptions import TaskFailed, TaskError
-
-# Exceptions
-class InvalidTask(Exception):
-    """Invalid task instance. User error on specifying the task."""
-    pass
-
+from doit.exceptions import InvalidTask, TaskFailed, TaskError
 
 
 # Actions
@@ -33,15 +30,16 @@ class CmdAction(BaseAction):
     @ivar task(Task): reference to task that contains this action
     """
 
-    def __init__(self, action):
-        assert isinstance(action,str), "CmdAction must be a string."
+    def __init__(self, action, task=None):
+        assert isinstance(action, str), "CmdAction must be a string."
         self.action = action
-        self.task = None
+        self.task = task
         self.out = None
         self.err = None
         self.result = None
         self.values = {}
 
+
     def execute(self, out=None, err=None):
         """
         Execute command action
@@ -63,10 +61,13 @@ class CmdAction(BaseAction):
         process = subprocess.Popen(action, shell=True,
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
-        def print_process_output(process, input, capture, realtime):
+        def print_process_output(process, input_, capture, realtime):
+            """read 'input_' untill process is terminated
+            write 'input_' content to 'capture' and 'realtime' streams
+            """
             while True:
                 # line buffered
-                line = input.readline()
+                line = input_.readline()
                 # unbuffered ? process.stdout.read(1)
                 if line:
                     capture.write(line)
@@ -131,16 +132,19 @@ class CmdAction(BaseAction):
 
 class Writer(object):
     """write to many streams"""
-    def __init__(self, *writers) :
+    def __init__(self, *writers):
+        """@param writers - file stream like objects"""
         self.writers = writers
 
-    def write(self, text) :
-        for w in self.writers :
-                w.write(text)
+    def write(self, text):
+        """write 'text' to all streams"""
+        for stream in self.writers:
+            stream.write(text)
 
-    def flush(self) :
-        for w in self.writers :
-                w.flush()
+    def flush(self):
+        """flush all streams"""
+        for stream in self.writers:
+            stream.flush()
 
 
 class PythonAction(BaseAction):
@@ -151,10 +155,10 @@ class PythonAction(BaseAction):
     @ivar kwargs: (dict) Extra keyword arguments to be passed to py_callable
     @ivar task(Task): reference to task that contains this action
     """
-    def __init__(self, py_callable, args=None, kwargs=None):
+    def __init__(self, py_callable, args=None, kwargs=None, task=None):
 
         self.py_callable = py_callable
-        self.task = None
+        self.task = task
         self.out = None
         self.err = None
         self.result = None
@@ -267,7 +271,7 @@ class PythonAction(BaseAction):
         try:
             # Python2.4
             try:
-                returned_value = self.py_callable(*self.args,**kwargs)
+                returned_value = self.py_callable(*self.args, **kwargs)
             # in python 2.4 SystemExit and KeyboardInterrupt subclass
             # from Exception.
             except (SystemExit, KeyboardInterrupt), exception:
@@ -307,7 +311,7 @@ class PythonAction(BaseAction):
         return "<PythonAction: '%s'>"% (repr(self.py_callable))
 
 
-def create_action(action):
+def create_action(action, task_ref):
     """
     Create action using proper constructor based on the parameter type
 
@@ -319,13 +323,17 @@ def create_action(action):
         return action
 
     if type(action) is str:
-        return CmdAction(action)
+        return CmdAction(action, task_ref)
 
     if type(action) is tuple:
-        return PythonAction(*action)
+        if len(action) > 3:
+            msg = "Task '%s': invalid 'actions' tuple length. got:%r %s"
+            raise InvalidTask(msg % (task_ref.name, action, type(action)))
+        py_callable, args, kwargs = (list(action) + [None]*(3-len(action)))
+        return PythonAction(py_callable, args, kwargs, task_ref)
 
     if hasattr(action, '__call__'):
-        return PythonAction(action)
+        return PythonAction(action, task=task_ref)
 
-    msg = "Invalid task action type. got %s"
-    raise InvalidTask(msg % (action.__class__))
+    msg = "Task '%s': invalid 'actions' type. got:%r %s"
+    raise InvalidTask(msg % (task_ref.name, action, type(action)))
