@@ -1,10 +1,12 @@
+"""doit CLI (command line interface)"""
+
 import os
 import sys
 import traceback
 
 import doit
-from doit import main
-from doit import task
+from doit.exceptions import InvalidDodoFile, InvalidCommand
+from doit import loader
 from doit import cmdparse
 from doit.cmds import doit_run, doit_clean, doit_list, doit_forget, doit_ignore
 from doit.cmds import doit_auto
@@ -116,7 +118,7 @@ opt_verbosity = {'name':'verbosity',
 opt_reporter = {'name':'reporter',
                  'short':'r',
                  'long':'reporter',
-                 'type':str, #TODO type choice
+                 'type':str, #TODO type choice (limit the accepted strings)
                  'default': 'default',
                  'help':
 """Choose output reporter. Available:
@@ -133,11 +135,13 @@ run_doc = {'purpose': "run tasks",
            'description': None}
 
 def print_version():
+    """print doit version (includes path location)"""
     print ".".join([str(i) for i in doit.__version__])
     print "bin @", os.path.abspath(__file__)
     print "lib @", os.path.dirname(os.path.abspath(doit.__file__))
 
 def print_usage():
+    """print doit "usage" (basic help) instructions"""
     # TODO cmd list should be automatically generated.
     print """
 doit -- automation tool
@@ -157,7 +161,7 @@ Commands:
 """
 
 def cmd_run(params, args):
-    """execute cmd run"""
+    """execute cmd 'run' """
 
     # special parameters that dont run anything
     if params["version"]:
@@ -170,8 +174,8 @@ def cmd_run(params, args):
 
     # check if no sub-command specified. default command is "run"
     if len(args) == 0 or args[0] not in params['sub']:
-        dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                    params['sub'].keys())
+        dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                      params['sub'].keys())
         params.update_defaults(dodo_tasks['config'])
         default_tasks = args or dodo_tasks['config'].get('default_tasks')
         return doit_run(params['dep_file'], dodo_tasks['task_list'],
@@ -228,12 +232,15 @@ opt_list_dependencies = {'name': 'list_deps',
                          'long': 'deps',
                          'type': bool,
                          'default': False,
-                         'help': "print list of dependencies (file dependencies only)"}
+                         'help': ("print list of dependencies "
+                                  "(file dependencies only)")
+}
 
 
 def cmd_list(params, args):
-    dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                params['sub'].keys())
+    """execute cmd 'list' """
+    dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                  params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     return doit_list(params['dep_file'], dodo_tasks['task_list'], sys.stdout,
                      args, params['all'], not params['quiet'],
@@ -262,8 +269,9 @@ opt_clean_cleandep = {'name': 'cleandep',
                     'help': 'clean task dependencies too'}
 
 def cmd_clean(params, args):
-    dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                params['sub'].keys())
+    """execute cmd 'clean' """
+    dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                  params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     options = args or dodo_tasks['config'].get('default_tasks')
     return doit_clean(dodo_tasks['task_list'], sys.stdout, params['dryrun'],
@@ -278,8 +286,9 @@ forget_doc = {'purpose': "clear successful run status from internal DB",
               'description': None}
 
 def cmd_forget(params, args):
-    dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                params['sub'].keys())
+    """execute cmd 'forget' """
+    dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                  params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     options = args or dodo_tasks['config'].get('default_tasks')
     return doit_forget(params['dep_file'], dodo_tasks['task_list'],
@@ -294,8 +303,9 @@ ignore_doc = {'purpose': "ignore task (skip) on subsequent runs",
               'description': None}
 
 def cmd_ignore(params, args):
-    dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                params['sub'].keys())
+    """execute cmd 'ignore' """
+    dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                  params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     return doit_ignore(params['dep_file'], dodo_tasks['task_list'],
                        sys.stdout, args)
@@ -309,8 +319,9 @@ auto_doc = {'purpose': "automatically execute tasks when a dependency changes",
             'description': None}
 
 def cmd_auto(params, args):
-    dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
-                                params['sub'].keys())
+    """execute cmd 'auto' """
+    dodo_tasks = loader.get_tasks(params['dodoFile'], params['cwdPath'],
+                                  params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     filter_tasks = args or dodo_tasks['config'].get('default_tasks')
     return doit_auto(params['dep_file'], dodo_tasks['task_list'], filter_tasks)
@@ -323,7 +334,9 @@ help_doc = {'purpose': "show help",
             'usage': "",
             'description': None}
 
+
 def print_task_help():
+    """print help for 'task' usage """
     print """
 
 Task Dictionary parameters
@@ -331,7 +344,7 @@ Task Dictionary parameters
 
 Tasks are defined by functions starting with the string ``task_``. It must return a dictionary describing the task with the following fields:
 
-action [required]:
+actions [required]:
   - type: Python-Task -> tuple (callable, `*args`, `**kwargs`)
   - type: Cmd-Task -> string or list of strings (each item is a different command). to be executed by shell.
   - type: Group-Task -> None.
@@ -339,27 +352,44 @@ action [required]:
 name [required for sub-task]:
   - type: string. sub-task identifier
 
-dependencies:
+file_dep:
   - type: list. items:
     * file (string) path relative to the dodo file
-    * task (string) ":<task_name>"
-    * TODO "?<task_name>"
+
+run_once:
+  - type: bool
     * run-once (True bool)
     * never up-to-date (False bool)
     * None values will be just ignored
 
+task_dep:
+  - type: list. items:
+    * task name (string)
+
+result_dep:
+  - type: list. items:
+    * task name (string)
+
+calc_dep:
+  - type: list. items:
+    * task name (string)
+
 getargs:
   - type: dictionary
     * key: string with the name of the function parater (used in a python-action)
     * value: string on the format <task-name>.<variable-name>
 
+setup:
+ - type: list. items:
+   * task name (string)
+
+teardown:
+ - type: (list) of actions (see above)
+
 targets:
   - type: list of strings
   - each item is file-path relative to the dodo file (accepts both files and folders)
 
-setup:
- - type: list of objects with methods 'setup' and 'cleanup'
-
 doc:
  - type: string -> the description text
 
@@ -386,6 +416,7 @@ title:
 """
 
 def cmd_help(params, args):
+    """execute cmd 'help' """
     if len(args) == 1:
         if args[0] in params['sub']:
             print params['sub'][args[0]].help()
@@ -402,62 +433,62 @@ def cmd_help(params, args):
 
 
 def cmd_main(cmd_args):
-    subCmd = {} # all sub-commands
+    """entry point for all commands"""
+    sub_cmd = {} # all sub-commands
 
     # help command
-    subCmd['help'] = cmdparse.Command('help', (), cmd_help, help_doc)
+    sub_cmd['help'] = cmdparse.Command('help', (), cmd_help, help_doc)
 
     # run command
     run_options = (opt_version, opt_help, opt_dodo, opt_cwd, opt_depfile,
                    opt_always, opt_continue, opt_verbosity, opt_reporter,
                    opt_outfile, opt_num_process)
-    subCmd['run'] = cmdparse.Command('run', run_options, cmd_run, run_doc)
+    sub_cmd['run'] = cmdparse.Command('run', run_options, cmd_run, run_doc)
 
     # clean command
     clean_options = (opt_dodo, opt_cwd, opt_clean_cleandep, opt_clean_dryrun)
-    subCmd['clean'] = cmdparse.Command('clean', clean_options, cmd_clean,
+    sub_cmd['clean'] = cmdparse.Command('clean', clean_options, cmd_clean,
                                        clean_doc)
 
     # list command
     list_options = (opt_dodo, opt_depfile, opt_cwd, opt_listall,
                     opt_list_quiet, opt_list_status, opt_list_private,
                     opt_list_dependencies)
-    subCmd['list'] = cmdparse.Command('list', list_options, cmd_list, list_doc)
+    sub_cmd['list'] = cmdparse.Command('list', list_options, cmd_list, list_doc)
 
     # forget command
     forget_options = (opt_dodo, opt_cwd, opt_depfile,)
-    subCmd['forget'] = cmdparse.Command('forget', forget_options,
+    sub_cmd['forget'] = cmdparse.Command('forget', forget_options,
                                         cmd_forget, forget_doc)
 
     # ignore command
     ignore_options = (opt_dodo, opt_cwd, opt_depfile,)
-    subCmd['ignore'] = cmdparse.Command('ignore', ignore_options,
+    sub_cmd['ignore'] = cmdparse.Command('ignore', ignore_options,
                                         cmd_ignore, ignore_doc)
 
     # auto command
     auto_options = (opt_dodo, opt_cwd, opt_depfile,)
-    subCmd['auto'] = cmdparse.Command('auto', auto_options,
+    sub_cmd['auto'] = cmdparse.Command('auto', auto_options,
                                         cmd_auto, auto_doc)
 
 
+
     try:
-        return subCmd['run'](cmd_args, sub=subCmd)
+        return sub_cmd['run'](cmd_args, sub=sub_cmd)
 
     # in python 2.4 SystemExit and KeyboardInterrupt subclass
     # from Exception.
-    # TODO maybe I should do something to help the user find out who
-    # is raising SystemExit. because it shouldnt happen...
     except (SystemExit, KeyboardInterrupt):
         raise
 
     # dont show traceback for user errors.
-    except (cmdparse.CmdParseError, main.InvalidDodoFile,
-            main.InvalidCommand, task.InvalidTask), err:
-        print "ERROR:", str(err)
+    # TODO check no InvalidTask exception endup here.
+    except (cmdparse.CmdParseError, InvalidDodoFile,
+            InvalidCommand), err:
+        sys.stderr.write("ERROR: %s\n" % str(err))
         return 1
 
-    # make sure exception is printed out. we migth have redirected stderr
     except Exception:
-        sys.__stderr__.write(traceback.format_exc())
+        sys.stderr.write(traceback.format_exc())
         return 1
 
