@@ -1,9 +1,9 @@
-"""Task runner."""
+"""Task runner"""
 
 import sys
 from multiprocessing import Process, Queue
 
-from doit.exceptions import CatchedException
+from doit.exceptions import InvalidTask, CatchedException
 from doit.exceptions import TaskFailed, SetupError, DependencyError
 from doit.dependency import Dependency
 from doit.task import Task
@@ -14,26 +14,28 @@ SUCCESS = 0
 FAILURE = 1
 ERROR = 2
 
+
 class Runner(object):
     """Task runner
 
-    run_tasks():
+    run_all()
+      run_tasks():
         for each task:
             select_task()
             execute_task()
             process_task_result()
-    finish()
+      finish()
 
     """
-    def __init__(self, dependencyFile, reporter, continue_=False,
+    def __init__(self, dependency_file, reporter, continue_=False,
                  always_execute=False, verbosity=0):
-        """@param dependencyFile: (string) file path of the db file
+        """@param dependency_file: (string) file path of the db file
         @param reporter: reporter to be used. It can be a class or an object
         @param continue_: (bool) execute all tasks even after a task failure
         @param always_execute: (bool) execute even if up-to-date or ignored
         @param verbosity: (int) 0,1,2 see Task.execute
         """
-        self.dependencyManager = Dependency(dependencyFile)
+        self.dependency_manager = Dependency(dependency_file)
         self.reporter = reporter
         self.continue_ = continue_
         self.always_execute = always_execute
@@ -45,8 +47,13 @@ class Runner(object):
 
 
     def _handle_task_error(self, task, catched_excp):
+        """handle all task failures/errors
+
+        called whenever there is an error before executing a task or
+        its execution is not successful.
+        """
         assert isinstance(catched_excp, CatchedException)
-        self.dependencyManager.remove_success(task)
+        self.dependency_manager.remove_success(task)
         self.reporter.add_failure(task, catched_excp)
         # only return FAILURE if no errors happened.
         if isinstance(catched_excp, TaskFailed):
@@ -61,30 +68,31 @@ class Runner(object):
         """Returns bool, task should be executed
          * side-effect: set task.options
 
-         Tasks should be executed if they are not up-to-date.
-         Tasks that cointains setup-tasks must be selected twice,
-         so it gives chance for other tasks to be executed after
-         checking it is not up-to-date.
+        Tasks should be executed if they are not up-to-date.
+
+        Tasks that cointains setup-tasks must be selected twice,
+        so it gives chance for dependency tasks to be executed after
+        checking it is not up-to-date.
         """
 
         # if run_status is not None, it was already calculated
         if task.run_status is None:
-            # TODO reporter.start_task rename to get_status
-            self.reporter.start_task(task)
+            self.reporter.get_status(task)
 
             # check if task is up-to-date
             try:
-                task.run_status = self.dependencyManager.get_status(task)
+                task.run_status = self.dependency_manager.get_status(task)
             except Exception, exception:
-                de = DependencyError("ERROR checking dependencies", exception)
-                self._handle_task_error(task, de)
+                msg = "ERROR: Task '%s' checking dependencies" % task.name
+                dep_error = DependencyError(msg, exception)
+                self._handle_task_error(task, dep_error)
                 return False
 
             if not self.always_execute:
                 # if task is up-to-date skip it
                 if task.run_status == 'up-to-date':
                     self.reporter.skip_uptodate(task)
-                    task.values = self.dependencyManager.get_values(task.name)
+                    task.values = self.dependency_manager.get_values(task.name)
                     return False
                 # check if task should be ignored (user controlled)
                 if task.run_status == 'ignore':
@@ -96,13 +104,14 @@ class Runner(object):
                 return False
         else:
             # sanity checks
-            assert task.run_status == 'run', "%s:%s" % (task.name, task.run_status)
+            assert task.run_status == 'run', \
+                "%s:%s" % (task.name, task.run_status)
             assert task.setup_tasks
 
         # selected just need to get values from other tasks
         for arg, value in task.getargs.iteritems():
             try:
-                task.options[arg] = self.dependencyManager.get_value(value)
+                task.options[arg] = self.dependency_manager.get_value(value)
             except Exception, exception:
                 msg = ("ERROR getting value for argument '%s'\n" % arg +
                        str(exception))
@@ -124,10 +133,11 @@ class Runner(object):
 
 
     def process_task_result(self, task, catched_excp):
+        """handles result"""
         task.run_status = "done"
         # save execution successful
         if catched_excp is None:
-            self.dependencyManager.save_success(task)
+            self.dependency_manager.save_success(task)
             self.reporter.add_success(task)
         # task error
         else:
@@ -166,9 +176,7 @@ class Runner(object):
     def finish(self):
         """finish running tasks"""
         # flush update dependencies
-        self.dependencyManager.close()
-
-        # new style teardown
+        self.dependency_manager.close()
         self.teardown()
 
         # report final results
@@ -176,14 +184,27 @@ class Runner(object):
         return self.final_result
 
 
+    def run_all(self, task_control):
+        """entry point to run tasks"""
+        # Python2.4 (remove double try)
+        try:
+            try:
+                self.run_tasks(task_control)
+            except InvalidTask, exception:
+                self.reporter.runtime_error(str(exception))
+        finally:
+            self.finish()
+        return self.final_result
+
+
 class Hold(object):
     """Sentinel class: No task ready to be executed"""
     pass
 
-class MP_Runner(Runner):
+class MRunner(Runner):
     """MultiProcessing Runner """
 
-    class MP_Reporter(object):
+    class MReporter(object):
         """send reported messages to master process
 
         puts a dictionary {'name': <task-name>,
@@ -204,16 +225,22 @@ class MP_Runner(Runner):
             return rep_method
 
 
-    def __init__(self, dependencyFile, reporter, continue_=False,
+    def __init__(self, dependency_file, reporter, continue_=False,
                  always_execute=False, verbosity=0, num_process=1):
-        Runner.__init__(self, dependencyFile, reporter, continue_,
+        Runner.__init__(self, dependency_file, reporter, continue_,
                         always_execute, verbosity)
         self.num_process = num_process
+
+        # key: name of task that has not completed yet
+        # value: list of task names that are waiting for task in key
         self.waiting = {}
+
+        # list of task names that are ready to be executed (these were waiting)
         self.ready_queue = []
-        self.free_proc = 0
-        self.task_gen = None
-        self.tasks = None
+        self.free_proc = 0   # number of free process
+        self.task_gen = None # genrator to retrieve tasks from controller
+        self.tasks = None    # dict of task instances by name
+        self.result_q = None
 
     def get_next_task(self):
         """get next task to be dispatched to sub-process
@@ -225,27 +252,27 @@ class MP_Runner(Runner):
         """
         if self._stop_running:
             return None # gentle stop
-
-        def nothing_ready():
-            if self.waiting:
-                self.free_proc += 1
-                return Hold()
-            else:
-                return None
-
         while True:
             # get new task from ready queue
             if self.ready_queue:
                 task_name = self.ready_queue.pop(0)
                 task = self.tasks[task_name]
+            # get next task from controller
             else:
                 try:
                     task = self.task_gen.next()
                     if not isinstance(task, Task):
                         self.free_proc += 1
                         return Hold()
+                # no more tasks from controller...
                 except StopIteration:
-                    return nothing_ready()
+                    # ... terminate one sub process if no other task waiting
+                    if not self.waiting:
+                        return None
+                    else:
+                        # extra-process might be used by waiting task, hold on
+                        self.free_proc += 1
+                        return Hold()
 
 
             # task with setup must be selected twice...
@@ -270,26 +297,36 @@ class MP_Runner(Runner):
                 if self.select_task(task):
                     return task
 
-    def set_tasks(self, task_control):
-        self.task_gen = task_control.task_dispatcher()
-        self.tasks = task_control.tasks
 
     def process_task_result(self, task, catched_excp):
+        """handle task result"""
         Runner.process_task_result(self, task, catched_excp)
         if task.name in self.waiting:
             for ready_task in self.waiting[task.name]:
                 self.ready_queue.append(ready_task)
             del self.waiting[task.name]
 
+
+    def _run_tasks_init(self, task_control):
+        """initialization for run_tasks"""
+        self.task_gen = task_control.task_dispatcher()
+        self.tasks = task_control.tasks
+
+
     def _run_start_processes(self, task_q, result_q):
+        """create and start sub-processes
+        @param task_q: (multiprocessing.Queue) tasks to be executed
+        @param result_q: (multiprocessing.Queue) collect task results
+        @return list of Process
+        """
         proc_list = []
-        for p_id in xrange(self.num_process):
+        for _ in xrange(self.num_process):
             next_task = self.get_next_task()
             if next_task is None:
                 break # do not start more processes than tasks
             if isinstance(next_task, Task):
                 task_q.put((next_task.name, next_task.file_dep))
-            else: # next_task is a string
+            else: # next_task is Hold
                 # no task ready to be executed but some are on the queue
                 # awaiting to be executed
                 task_q.put((next_task, None))
@@ -299,16 +336,21 @@ class MP_Runner(Runner):
             proc_list.append(process)
         return proc_list
 
+
     def run_tasks(self, task_control):
+        """controls subprocesses task dispatching and result collection
+        """
+        # result queue - result collected from sub-processes
         result_q = Queue()
+        # task queue - tasks ready to be dispatched to sub-processes
         task_q = Queue()
-        self.set_tasks(task_control)
-        # create and start processes
+        self._run_tasks_init(task_control)
         proc_list = self._run_start_processes(task_q, result_q)
 
         # wait for all processes terminate
         proc_count = len(proc_list)
         while proc_count:
+            # wait until there is a result to be consumed
             result = result_q.get()
             task = task_control.tasks[result['name']]
             if 'reporter' in result:
@@ -326,9 +368,11 @@ class MP_Runner(Runner):
             # completed one task, dispatch next one
             self.process_task_result(task, catched_excp)
 
-            free_proc = self.free_proc
+            # update num free process
+            free_proc = self.free_proc + 1
             self.free_proc = 0
-            for get_one_more in range(1 + free_proc):
+            # tries to get as many tasks as free process
+            for _ in range(free_proc):
                 next_task = self.get_next_task()
                 if next_task is None:
                     proc_count -= 1
@@ -336,20 +380,8 @@ class MP_Runner(Runner):
                     task_q.put((next_task.name, next_task.file_dep))
                 else:
                     task_q.put((next_task, None))
-
-
             # check for cyclic dependencies
-            if len(proc_list) == self.free_proc:
-                msg = "Cyclic dependencies on tasks:\n"
-                for task_name, wait_list in self.waiting.iteritems():
-                    msg += (" * Task %s is waiting for tasks %s\n" %
-                            (task_name, ", ".join(wait_list)))
-                self.final_result = ERROR
-                self.reporter.runtime_error(msg)
-                # terminate all child process
-                proc_count = 0
-                for proc in proc_list:
-                    task_q.put((None, None))
+            assert len(proc_list) > self.free_proc
 
         # we are done, join all process
         for proc in proc_list:
@@ -364,10 +396,14 @@ class MP_Runner(Runner):
 
 
     def execute_task(self, task_q, result_q):
-        """executed on child processes"""
+        """executed on child processes
+        @param task_q: task queue,
+            * None elements indicate process can terminate
+            * Hold indicate process should wait for next task
+            * Task task to be executed
+        """
         self.result_q = result_q
-        self.reporter = self.MP_Reporter(self, self.reporter)
-
+        self.reporter = self.MReporter(self, self.reporter)
         try:
             while True:
                 task_name, file_dep = task_q.get()
@@ -375,13 +411,16 @@ class MP_Runner(Runner):
                     self.teardown()
                     return # no more tasks to execute finish this process
 
+                # do nothing. this used to start the subprocess even if no task
+                # is available when process is created.
                 if isinstance(task_name, Hold):
                     continue
 
                 task = self.tasks[task_name]
+                # FIXME why is this required? to update calc-deps?
+                # what about other dependencies?
                 task.file_dep = file_dep
                 result = {'name': task.name}
-                # FIXME support setup objects with 2 "scopes" (global and process)
                 t_result = Runner.execute_task(self, task)
 
                 if t_result is None:
@@ -390,8 +429,9 @@ class MP_Runner(Runner):
                 else:
                     result['failure'] = t_result
                 result_q.put(result)
-        except (SystemExit, KeyboardInterrupt, Exception), e:
-            # error, blow-up everything
+        except (SystemExit, KeyboardInterrupt, Exception), exception:
+            # error, blow-up everything. send exception info to master process
             result_q.put({'name': task_name,
-                          'exit': e.__class__,
-                          'exception': str(e)})
+                          'exit': exception.__class__,
+                          'exception': str(exception)})
+
