@@ -1,9 +1,11 @@
+"""Tasks are the main abstractions managed by doit"""
+
 import types
 import os
 
 from doit import cmdparse
-from doit.exceptions import CatchedException
-from doit.action import create_action, InvalidTask
+from doit.exceptions import CatchedException, InvalidTask
+from doit.action import create_action
 
 
 class Task(object):
@@ -16,13 +18,15 @@ class Task(object):
     @ivar targets: (list -string)
     @ivar task_dep: (list - string)
     @ivar wild_dep: (list - string) task dependency using wildcard *
-    @ivar file_dep: (list - string)
+    @ivar file_dep: (set - string)
+    @ivar result_dep: (list -string)
     @ivar calc_dep: (list - string) reference to a task
     @ivar calc_dep_stack: (list - string) unprocessed calc_dep
     @ivar dep_changed (list - string): list of file-dependencies that changed
           (are not up_to_date). this must be set before
     @ivar run_once: (bool) task without dependencies should run
-    @ivar run_always: (bool) task always run even if up-to-date
+    @ivar uptodate: (list - bool/None) use bool/computed value instead of
+                                       checking dependencies
     @ivar setup_tasks (list - string): references to task-names
     @ivar is_subtask: (bool) indicate this task is a subtask.
     @ivar result: (str) last action "result". used to check task-result-dep
@@ -35,35 +39,42 @@ class Task(object):
     @ivar custom_title: function reference that takes a task object as
                         parameter and returns a string.
     @ivar run_status (str): contains the result of Dependency.get_status
-            modified by runner, value can be: None, run, ignore, up-to-date
+            modified by runner, value can be:
+           - None: not processed yet
+           - run: task is selected to be executed (it might be running or
+                   waiting for setup)
+           - ignore: task wont be executed (user forced deselect)
+           - up-to-date: task wont be executed (no need)
+           - done: task finished its execution
     """
 
     DEFAULT_VERBOSITY = 1
 
     # list of valid types/values for each task attribute.
-    valid_attr = {'name': [str],
-                  'actions': [list, tuple, None],
-                  'file_dep': [list, tuple],
-                  'task_dep': [list, tuple],
-                  'result_dep': [list, tuple],
-                  'calc_dep': [list, tuple],
-                  'targets': [list, tuple],
-                  'setup': [list, tuple],
-                  'clean': [list, tuple, True],
-                  'teardown': [list, tuple],
-                  'doc': [str, None],
-                  'params': [list, tuple],
-                  'verbosity': [None,0,1,2],
-                  'getargs': [dict],
-                  'title': [None, types.FunctionType],
+    valid_attr = {'name': ([str], []),
+                  'actions': ([list, tuple], [None]),
+                  'file_dep': ([list, tuple], []),
+                  'task_dep': ([list, tuple], []),
+                  'result_dep': ([list, tuple], []),
+                  'uptodate': ([list, tuple], []),
+                  'run_once': ([bool], []),
+                  'calc_dep': ([list, tuple], []),
+                  'targets': ([list, tuple], []),
+                  'setup': ([list, tuple], []),
+                  'clean': ([list, tuple], [True]),
+                  'teardown': ([list, tuple], []),
+                  'doc': ([str], [None]),
+                  'params': ([list, tuple], []),
+                  'verbosity': ([], [None,0,1,2]),
+                  'getargs': ([dict], []),
+                  'title': ([types.FunctionType], [None]),
                   }
 
-    setup_warned = False # print setup-object deprecation warning just once
 
     def __init__(self, name, actions, file_dep=(), targets=(),
-                 task_dep=(), result_dep=(), calc_dep=(),
-                 setup=(), clean=(), teardown=(), is_subtask=False, doc=None,
-                 params=(), verbosity=None, title=None, getargs=None):
+                 task_dep=(), result_dep=(), uptodate=(), run_once=False,
+                 calc_dep=(), setup=(), clean=(), teardown=(), is_subtask=False,
+                 doc=None, params=(), verbosity=None, title=None, getargs=None):
         """sanity checks and initialization
 
         @param params: (list of option parameters) see cmdparse.Command.__init__
@@ -71,13 +82,14 @@ class Task(object):
 
         getargs = getargs or {} #default
         # check task attributes input
+        my_locals = locals()
         for attr, valid_list in self.valid_attr.iteritems():
-            self.check_attr_input(name, attr, locals()[attr], valid_list)
+            self.check_attr_input(name, attr, my_locals[attr], valid_list)
 
         self.name = name
         self.targets = targets
-        self.run_once = False
-        self.run_always = False
+        self.uptodate = list(uptodate)
+        self.run_once = run_once
         self.is_subtask = is_subtask
         self.result = None
         self.values = {}
@@ -96,7 +108,7 @@ class Task(object):
         if actions is None:
             self.actions = []
         else:
-            self.actions = [create_action(a) for a in actions]
+            self.actions = [create_action(a, self) for a in actions]
 
         # clean
         if clean is True:
@@ -104,19 +116,16 @@ class Task(object):
             self.clean_actions = ()
         else:
             self._remove_targets = False
-            self.clean_actions = [create_action(a) for a in clean]
+            self.clean_actions = [create_action(a, self) for a in clean]
 
         # teardown
-        self.teardown = [create_action(a) for a in teardown]
-
-        # set self as task for all actions
-        for action in self.actions:
-            action.task = self
+        self.teardown = [create_action(a, self) for a in teardown]
 
         # dependencies
         self.dep_changed = None
 
-        self.file_dep = []
+        # file_dep
+        self.file_dep = set()
         self._expand_file_dep(file_dep)
 
         # task_dep
@@ -134,27 +143,45 @@ class Task(object):
         self._expand_calc_dep(calc_dep)
 
         self._init_getargs()
-        self._init_doc(doc)
+        self.doc = self._init_doc(doc)
 
         self.run_status = None
 
+
     def _expand_file_dep(self, file_dep):
+        """put input into file_dep"""
         for dep in file_dep:
-            # bool
-            if isinstance(dep,bool):
-                if dep is True:
-                    self.run_once = True
-                if dep is False:
-                    self.run_always = True
-            # ignore None values
-            elif dep is None:
+
+            # deprecation to be removed on 0.11
+            if dep is True:
+                self.run_once = True
+                print "DEPRECATION WARNING: task %s" % self.name
+                print "file_dep can not be True, use run_once"
                 continue
-            else:
-                # TODO store file_deps in a set
-                if dep not in self.file_dep:
-                    self.file_dep.append(dep)
+
+            if dep in (False, None):
+                self.uptodate.append(dep)
+                print "DEPRECATION WARNING: task %s" % self.name
+                print "file_dep can not be %s, use uptodate" % repr(dep)
+                continue
+            # end - deprecation to be removed on 0.11
+
+            if not isinstance(dep, str):
+                raise InvalidTask("%s. file_dep must be a str got '%r' (%s)" %
+                                  (self.name, dep, type(dep)))
+
+            if dep not in self.file_dep:
+                self.file_dep.add(dep)
+
+        # run_once can't be used together with file dependencies
+        if self.run_once and self.file_dep:
+            msg = ("%s. task cant have file and dependencies and run_once " +
+                   "at the same time. (just remove run_once)")
+            raise InvalidTask(msg % self.name)
+
 
     def _expand_task_dep(self, task_dep):
+        """convert task_dep input into actaul task_dep and wild_dep"""
         for dep in task_dep:
             if "*" in dep:
                 self.wild_dep.append(dep)
@@ -162,18 +189,22 @@ class Task(object):
                 self.task_dep.append(dep)
 
     def _expand_result_dep(self, result_dep):
+        """convert ressult_dep input into restul_dep and task_dep"""
         self.result_dep.extend(result_dep)
         # result_dep are also task_dep
-        self.task_dep.extend(self.result_dep)
+        self.task_dep.extend(result_dep)
+
 
     def _expand_calc_dep(self, calc_dep):
+        """calc_dep input"""
         for dep in calc_dep:
             if dep not in self.calc_dep:
-                self.calc_dep.extend(calc_dep)
-                self.calc_dep_stack.extend(calc_dep)
+                self.calc_dep.append(dep)
+                self.calc_dep_stack.append(dep)
 
 
     def update_deps(self, deps):
+        """expand all kinds of dep input"""
         for dep, dep_values in deps.iteritems():
             if dep == 'task_dep':
                 self._expand_task_dep(dep_values)
@@ -183,10 +214,11 @@ class Task(object):
                 self._expand_result_dep(dep_values)
             elif dep == 'calc_dep':
                 self._expand_calc_dep(dep_values)
-
+            elif dep == 'uptodate':
+                self.uptodate.extend(dep_values)
 
     def _init_getargs(self):
-        # getargs also define implicit task dependencies
+        """task getargs attribute define implicit task dependencies"""
         for key, desc in self.getargs.iteritems():
             # check format
             parts = desc.split('.')
@@ -198,24 +230,17 @@ class Task(object):
             if parts[0] not in self.task_dep:
                 self.task_dep.append(parts[0])
 
-        # run_once can't be used together with file dependencies
-        if self.run_once and self.file_dep:
-            msg = ("%s. task cant have file and dependencies and True " +
-                   "at the same time. (just remove True)")
-            raise InvalidTask(msg % self.name)
 
-    def _init_doc(self, doc):
-        # Store just first non-empty line as documentation string
-        if doc is None:
-            self.doc = ''
-        else:
+    @staticmethod
+    def _init_doc(doc):
+        """process task "doc" attribute"""
+        # store just first non-empty line as documentation string
+        if doc is not None:
             for line in doc.splitlines():
                 striped = line.strip()
                 if striped:
-                    self.doc = striped
-                    break
-            else:
-                self.doc = ''
+                    return striped
+        return ''
 
 
     @staticmethod
@@ -228,24 +253,22 @@ class Task(object):
         @param valid (list): of valid types/value accepted
         @raises InvalidTask if invalid input
         """
-        msg = "Task %s attribute '%s' must be {%s} got:%r %s"
-        for expected in valid:
-            # check expected type
-            if isinstance(expected, type):
-                if isinstance(value, expected):
-                    return
-            # check expected value
-            else:
-                if expected is value:
-                    return
+        value_type = type(value)
+        if value_type in valid[0]:
+            return
+        if value in valid[1]:
+            return
 
         # input value didnt match any valid type/value, raise execption
-        accept = ", ".join([getattr(v,'__name__',str(v)) for v in valid])
-        raise InvalidTask(msg % (task, attr, accept, str(value), type(value)))
+        msg = "Task '%s' attribute '%s' must be " % (task, attr)
+        accept = ", ".join([getattr(v, '__name__', str(v)) for v in
+                            (valid[0] + valid[1])])
+        msg += "{%s} got:%r %s" % (accept, value, type(value))
+        raise InvalidTask(msg)
 
 
     def _get_out_err(self, out, err, verbosity):
-        # select verbosity to be used
+        """select verbosity to be used"""
         priority = (verbosity, # use command line option
                     self.verbosity, # or task default from dodo file
                     self.DEFAULT_VERBOSITY) # or global default
@@ -289,8 +312,8 @@ class Task(object):
         """
         # if clean is True remove all targets
         if self._remove_targets is True:
-            files = filter(os.path.isfile, self.targets)
-            dirs = filter(os.path.isdir, self.targets)
+            files = [path for path in self.targets if os.path.isfile(path)]
+            dirs = [path for path in self.targets if os.path.isdir(path)]
 
             # remove all files
             for file_ in files:
@@ -303,7 +326,7 @@ class Task(object):
             for dir_ in dirs:
                 if os.listdir(dir_):
                     msg = "%s - cannot remove (it is not empty) '%s'\n"
-                    outstream.write(msg % (self.name, file_))
+                    outstream.write(msg % (self.name, dir_))
                 else:
                     msg = "%s - removing dir '%s'\n"
                     outstream.write(msg % (self.name, dir_))
@@ -333,7 +356,6 @@ class Task(object):
         return "<Task: %s>"% self.name
 
 
-DEPRECATION = False # already printed deprecation?
 def dict_to_task(task_dict):
     """Create a task instance from dictionary.
 
@@ -343,8 +365,6 @@ def dict_to_task(task_dict):
     @param task_dict (dict): task representation as a dict.
     @raise InvalidTask: If unexpected fields were passed in task_dict
     """
-    global DEPRECATION
-
     # check required fields
     if 'actions' not in task_dict:
         raise InvalidTask("Task %s must contain 'actions' field. %s" %
@@ -352,34 +372,9 @@ def dict_to_task(task_dict):
 
     # user friendly. dont go ahead with invalid input.
     task_attrs = task_dict.keys()
+    valid_attrs = set(Task.valid_attr.iterkeys())
     for key in task_attrs:
-
-        if key == 'dependencies':
-            if not DEPRECATION:
-                print ("DEPRECATION WARNING: task attribute 'dependencies': "
-                       "use 'file_dep', 'task_dep', 'result_dep' or 'calc_dep'.")
-                DEPRECATION = True
-            task_dict['file_dep'] = []
-            task_dict['task_dep'] = []
-            task_dict['result_dep'] = []
-            task_dict['calc_dep'] = []
-            for dep in task_dict['dependencies']:
-                if isinstance(dep, str) or isinstance(dep, unicode):
-                    # task dep starts with a ':'
-                    if dep.startswith(':'):
-                        task_dict['task_dep'].append(dep[1:])
-                    # task-result dep starts with a '?'
-                    elif dep.startswith('?'):
-                        task_dict['result_dep'].append(dep[1:])
-                    # file dep
-                    else:
-                        task_dict['file_dep'].append(dep)
-                else:
-                    # bool or None
-                    task_dict['file_dep'].append(dep)
-            del task_dict['dependencies']
-
-        elif key not in Task.valid_attr.keys():
+        if key not in valid_attrs:
             raise InvalidTask("Task %s contains invalid field: '%s'"%
                               (task_dict['name'],key))
 
