@@ -19,6 +19,13 @@ def pytest_funcarg__tmpfile(request):
         teardown=(lambda tmpfile: tmpfile.close()),
         scope="function")
 
+class FakeTask(object):
+    def __init__(self, file_dep, dep_changed, targets, options):
+        self.name = "Fake"
+        self.file_dep = file_dep
+        self.dep_changed = dep_changed
+        self.targets = targets
+        self.options = options
 
 
 ############# CmdAction
@@ -97,6 +104,32 @@ class TestCmdVerbosity(object):
         assert "hi_stdout" == my_action.out, repr(my_action.out)
 
 
+class TestCmdExpandAction(object):
+
+    def test_task_meta_reference(self):
+        cmd = "python %s/myecho.py" % TEST_PATH
+        cmd += " %(dependencies)s - %(changed)s - %(targets)s"
+        dependencies = ["data/dependency1", "data/dependency2", ":dep_on_task"]
+        targets = ["data/target", "data/targetXXX"]
+        task = FakeTask(dependencies, ["data/dependency1"], targets, {})
+        my_action = action.CmdAction(cmd, task)
+        my_action.execute()
+
+        got = my_action.out.split('-')
+        assert task.file_dep == got[0].split(), got[0]
+        assert task.dep_changed == got[1].split(), got[1]
+        assert targets == got[2].split(), got[2]
+
+    def test_task_options(self):
+        cmd = "python %s/myecho.py" % TEST_PATH
+        cmd += " %(opt1)s - %(opt2)s"
+        task = FakeTask([],[],[],{'opt1':'3', 'opt2':'abc def'})
+        my_action = action.CmdAction(cmd, task)
+        my_action.execute()
+
+        got = my_action.out.strip()
+        assert "3 - abc def" == got, repr(got)
+
 
 class TestWriter(object):
     def test_writer(self):
@@ -280,32 +313,130 @@ class TestPythonVerbosity(object):
         assert "this is stdout S\n" == got, got
 
 
+class TestPythonActionPrepareKwargsMeta(object):
+
+    def pytest_funcarg__task_depchanged(self, request):
+        def create_task_depchanged():
+            task = FakeTask(['dependencies'],['changed'],['targets'],{})
+            return task
+        return request.cached_setup(
+            setup=create_task_depchanged,
+            scope="function")
+
+    def test_no_extra_args(self, task_depchanged):
+        def py_callable():
+            return True
+        my_action = action.PythonAction(py_callable, task=task_depchanged)
+        my_action.execute()
+
+    def test_keyword_extra_args(self, task_depchanged):
+        got = []
+        def py_callable(arg=None, **kwargs):
+            got.append(kwargs['targets'])
+            got.append(kwargs['dependencies'])
+            got.append(kwargs['changed'])
+        my_action = action.PythonAction(py_callable, task=task_depchanged)
+        my_action.execute()
+        assert got == [['targets'], ['dependencies'], ['changed']], got
+
+    def test_named_extra_args(self, task_depchanged):
+        got = []
+        def py_callable(targets, dependencies, changed):
+            got.append(targets)
+            got.append(dependencies)
+            got.append(changed)
+        my_action = action.PythonAction(py_callable, task=task_depchanged)
+        my_action.execute()
+        assert got == [['targets'], ['dependencies'], ['changed']]
+
+    def test_mixed_args(self, task_depchanged):
+        got = []
+        def py_callable(a, b, changed):
+            got.append(a)
+            got.append(b)
+            got.append(changed)
+        my_action = action.PythonAction(py_callable, ('a', 'b'),
+                                        task=task_depchanged)
+        my_action.execute()
+        assert got == ['a', 'b', ['changed']]
+
+    def test_extra_arg_overwritten(self, task_depchanged):
+        got = []
+        def py_callable(a, b, changed):
+            got.append(a)
+            got.append(b)
+            got.append(changed)
+        my_action = action.PythonAction(py_callable, ('a', 'b', 'c'),
+                                        task=task_depchanged)
+        my_action.execute()
+        assert got == ['a', 'b', 'c']
+
+    def test_extra_kwarg_overwritten(self, task_depchanged):
+        got = []
+        def py_callable(a, b, **kwargs):
+            got.append(a)
+            got.append(b)
+            got.append(kwargs['changed'])
+        my_action = action.PythonAction(py_callable, ('a', 'b'),
+                                        {'changed': 'c'}, task_depchanged)
+        my_action.execute()
+        assert got == ['a', 'b', 'c']
+
+    def test_extra_arg_default_disallowed(self, task_depchanged):
+        def py_callable(a, b, changed=None): pass
+        my_action = action.PythonAction(py_callable, ('a', 'b'),
+                                        task=task_depchanged)
+        py.test.raises(action.InvalidTask, my_action.execute)
+
+class TestPythonActionOptions(object):
+    def test_task_options(self):
+        got = []
+        def py_callable(opt1, opt3):
+            got.append(opt1)
+            got.append(opt3)
+        task = FakeTask([],[],[],{'opt1':'1', 'opt2':'abc def', 'opt3':3})
+        my_action = action.PythonAction(py_callable, task=task)
+        my_action.execute()
+        assert ['1',3] == got, repr(got)
+
+
 ##############
 
 
 class TestCreateAction(object):
+    class TaskStub(object):
+        name = 'stub'
+    mytask = TaskStub()
+
     def testBaseAction(self):
         class Sample(action.BaseAction): pass
-        my_action = action.create_action(Sample())
+        my_action = action.create_action(Sample(), self.mytask)
         assert isinstance(my_action, Sample)
 
     def testStringAction(self):
-        my_action = action.create_action("xpto 14 7")
+        my_action = action.create_action("xpto 14 7", self.mytask)
         assert isinstance(my_action, action.CmdAction)
 
     def testMethodAction(self):
         def dumb(): return
-        my_action = action.create_action(dumb)
+        my_action = action.create_action(dumb, self.mytask)
         assert isinstance(my_action, action.PythonAction)
 
     def testTupleAction(self):
         def dumb(): return
-        my_action = action.create_action((dumb,[1,2],{'a':5}))
+        my_action = action.create_action((dumb,[1,2],{'a':5}), self.mytask)
         assert isinstance(my_action, action.PythonAction)
 
+    def testTupleActionMoreThanThreeElements(self):
+        def dumb(): return
+        py.test.raises(action.InvalidTask, action.create_action,
+                       (dumb,[1,2],{'a':5},'oo'), self.mytask)
+
     def testInvalidActionNone(self):
-        py.test.raises(action.InvalidTask, action.create_action, None)
+        py.test.raises(action.InvalidTask, action.create_action,
+                       None, self.mytask)
 
     def testInvalidActionObject(self):
-        py.test.raises(action.InvalidTask, action.create_action, self)
+        py.test.raises(action.InvalidTask, action.create_action,
+                       self, self.mytask)
 
