@@ -1,10 +1,12 @@
 import os
 import time
+import anydbm
 
 import py.test
 
 from doit.task import Task
-from doit.dependency import get_md5, md5sum, check_modified, Dependency
+from doit.dependency import get_md5, md5sum, check_modified
+from doit.dependency import JsonDependency, DbmDependency
 
 
 def get_abspath(relativePath):
@@ -37,11 +39,12 @@ def test_md5():
 
 TESTDB = "testdb"
 
-
+# fixture for "doit.db". create/remove for every test
 def pytest_funcarg__depfile(request):
     def create_depfile():
+        dep_class = request.param
         if os.path.exists(TESTDB): os.remove(TESTDB)
-        return Dependency(TESTDB)
+        return dep_class(TESTDB)
     def remove_depfile(depfile):
         if not depfile._closed:
             depfile.close()
@@ -53,6 +56,7 @@ def pytest_funcarg__depfile(request):
         scope="function")
 
 
+# fixture to create a sample file to be used as file_dep
 def pytest_funcarg__dependency(request):
     def create_dependency():
         path = get_abspath("data/dependency1")
@@ -70,6 +74,16 @@ def pytest_funcarg__dependency(request):
         scope="function")
 
 
+# FIXME there was major refactor breaking classes from dependency,
+# unit-tests could be more specific to base classes.
+
+# test parametrization, execute tests for all DB backends
+def pytest_generate_tests(metafunc):
+    if "depfile" in metafunc.funcargnames:
+        metafunc.addcall(id='JsonDependency', param=JsonDependency)
+        metafunc.addcall(id='DbmDependency', param=DbmDependency)
+
+
 class TestDependencyDb(object):
 
     # adding a new value to the DB
@@ -79,21 +93,21 @@ class TestDependencyDb(object):
         assert "da_md5" == value, value
 
     #
-    def test_setPersistence(self, depfile):
+    def test_dump(self, depfile):
         # save and close db
         depfile._set("taskId_X","dependency_A","da_md5")
         depfile.close()
 
         # open it again and check the value
-        d2 = Dependency(TESTDB)
+        d2 = depfile.__class__(TESTDB)
         value = d2._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
 
-    def test_corrupted_file(self):
+    def test_corrupted_file(self, depfile):
         fd = open(TESTDB, 'w')
         fd.write("""{"x": y}""")
         fd.close()
-        py.test.raises(ValueError, Dependency, TESTDB)
+        py.test.raises((ValueError, anydbm.error), depfile.__class__, TESTDB)
 
 
     # _get must return None if entry doesnt exist.
@@ -101,6 +115,12 @@ class TestDependencyDb(object):
         assert depfile._get("taskId_X","dependency_A") == None
 
 
+    def test_in(self, depfile):
+        depfile._set("taskId_ZZZ","dep_1","12")
+        assert depfile._in("taskId_ZZZ")
+        assert not depfile._in("taskId_hohoho")
+
+
     def test_remove(self, depfile):
         depfile._set("taskId_ZZZ","dep_1","12")
         depfile._set("taskId_ZZZ","dep_2","13")
@@ -111,6 +131,22 @@ class TestDependencyDb(object):
         assert "14" == depfile._get("taskId_YYY","dep_1")
 
 
+    # special test for DBM backend and "dirty"/caching mechanism
+    def test_remove_from_non_empty_file(self, depfile):
+        # 1 - put 2 tasks of file
+        depfile._set("taskId_XXX", "dep_1", "x")
+        depfile._set("taskId_YYY", "dep_1", "x")
+        depfile.close()
+        # 2 - re-open and remove one task
+        reopened = depfile.__class__(TESTDB)
+        reopened.remove("taskId_YYY")
+        reopened.close()
+        # 3 - re-open again and check task was really removed
+        reopened2 = depfile.__class__(TESTDB)
+        assert reopened2._in("taskId_XXX")
+        assert not reopened2._in("taskId_YYY")
+
+
     def test_remove_all(self, depfile):
         depfile._set("taskId_ZZZ","dep_1","12")
         depfile._set("taskId_ZZZ","dep_2","13")
@@ -121,6 +157,7 @@ class TestDependencyDb(object):
         assert None == depfile._get("taskId_YYY","dep_1")
 
 
+
 class TestSaveSuccess(object):
 
     def test_save_result(self, depfile):
@@ -135,7 +172,7 @@ class TestSaveSuccess(object):
         assert None is depfile._get(t1.name, "result:")
 
     def test_save_runonce(self, depfile):
-        t1 = Task('t_name', None, [True])
+        t1 = Task('t_name', None, run_once=True)
         depfile.save_success(t1)
         assert depfile._get(t1.name, "run-once:")
 
@@ -176,7 +213,7 @@ class TestSaveSuccess(object):
         ff.write("part2")
         ff.close()
 
-        assert 0 == len(depfile._db)
+        assert 0 == len(depfile.backend._db)
         t1 = Task("taskId_X", None, [filePath,filePath2])
         depfile.save_success(t1)
         assert depfile._get("taskId_X",filePath) is not None
@@ -313,6 +350,12 @@ class TestGetStatus(object):
         assert dependencies == t1.dep_changed
 
 
+    def test_file_dependency_not_exist(self, depfile):
+        filePath = get_abspath("data/dependency_not_exist")
+        t1 = Task("t1", None, [filePath])
+        py.test.raises(Exception, depfile.get_status, t1)
+
+
     # if there is no dependency the task is always executed
     def test_noDependency(self, depfile):
         t1 = Task("t1", None)
@@ -326,21 +369,20 @@ class TestGetStatus(object):
 
 
     def test_runOnce(self, depfile):
-        t1 = Task("t1", None, [True])
+        t1 = Task("t1", None, run_once=True)
         assert 'run' == depfile.get_status(t1)
         assert [] == t1.dep_changed
         depfile.save_success(t1)
         assert 'up-to-date' == depfile.get_status(t1)
         assert [] == t1.dep_changed
 
-    def test_runAlways(self, depfile):
+    def test_UptodateFalse(self, depfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
         ff.close()
 
-        dependencies = [filePath, False]
-        t1 = Task("t1", None, dependencies)
+        t1 = Task("t1", None, file_dep=[filePath], uptodate=[False])
 
         # first time execute
         assert 'run' == depfile.get_status(t1)
@@ -351,6 +393,27 @@ class TestGetStatus(object):
         assert 'run' == depfile.get_status(t1)
         assert [] == t1.dep_changed
 
+    def test_UptodateTrue(self, depfile):
+        t1 = Task("t1", None, uptodate=[True])
+        depfile.save_success(t1)
+        assert 'up-to-date' == depfile.get_status(t1)
+
+    def test_UptodateNone(self, depfile):
+        filePath = get_abspath("data/dependency1")
+        ff = open(filePath,"w")
+        ff.write("part1")
+        ff.close()
+
+        t1 = Task("t1", None, file_dep=[filePath], uptodate=[None])
+
+        # first time execute
+        assert 'run' == depfile.get_status(t1)
+        assert [filePath] == t1.dep_changed
+
+        # second time execute too
+        depfile.save_success(t1)
+        assert 'up-to-date' == depfile.get_status(t1)
+
 
     # if target file does not exist, task is outdated.
     def test_targets_notThere(self, depfile, dependency):
