@@ -4,8 +4,9 @@ import inspect
 import py.test
 from mock import Mock
 
+from doit.exceptions import InvalidCommand
 from doit import doit_cmd
-from doit import main
+from doit import loader
 
 
 def mock_get_tasks(*args, **kwargs):
@@ -41,14 +42,14 @@ class TestRun(object):
         assert "Purpose: list tasks from dodo file" in out
 
     def test_run_is_default(self, monkeypatch):
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_run = Mock()
         monkeypatch.setattr(doit_cmd, "doit_run", mock_run)
         doit_cmd.cmd_main([])
         assert 1 == mock_run.call_count
 
     def test_run_other_subcommand(self, monkeypatch):
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_list = Mock()
         monkeypatch.setattr(doit_cmd, "doit_list", mock_list)
         doit_cmd.cmd_main(["list"])
@@ -58,18 +59,18 @@ class TestRun(object):
 class TestInterface(object):
     def test_doit_run_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_run)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_run = Mock()
         monkeypatch.setattr(doit_cmd, "doit_run", mock_run)
         doit_cmd.cmd_main(["--output-file", "mylog.txt",
                          "-v2", "--continue", "--reporter", "myr", "-n", "4"])
 
-        expected = [('dependencyFile', '.doit.db'),
+        expected = [('dependency_file', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
                     ('output', 'mylog.txt'),
                     ('options', mock_get_tasks()['config']['default_tasks']),
                     ('verbosity', 2),
-                    ('alwaysExecute', False),
+                    ('always_execute', False),
                     ('continue_', True),
                     ('reporter', "myr"),
                     ('num_process', 4),]
@@ -83,7 +84,7 @@ class TestInterface(object):
         get_tasks_result = {'task_list': ['a','b','c'],
                             'config': {'reporter': 'config_reporter',
                                        'outfile': 'config_outfile'}}
-        monkeypatch.setattr(main, "get_tasks",
+        monkeypatch.setattr(loader, "get_tasks",
                             Mock(return_value=get_tasks_result))
         argspec = inspect.getargspec(doit_cmd.doit_run)
         mock_run = Mock()
@@ -103,14 +104,14 @@ class TestInterface(object):
 
     def test_doit_list_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_list)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_list = Mock()
         monkeypatch.setattr(doit_cmd, "doit_list", mock_list)
         doit_cmd.cmd_main(["list", "--all", "--quiet", "--status", "--deps",
                            "c", "a"])
         assert mock_list.called
 
-        expected = [('dependencyFile', '.doit.db'),
+        expected = [('dependency_file', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
                     ('outstream', sys.stdout),
                     ('filter_tasks', ["c", "a"]),
@@ -128,7 +129,7 @@ class TestInterface(object):
 
     def test_doit_clean_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_clean)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_clean = Mock()
         monkeypatch.setattr(doit_cmd, "doit_clean", mock_clean)
         doit_cmd.cmd_main(["clean", "--dry-run"])
@@ -149,15 +150,15 @@ class TestInterface(object):
 
     def test_doit_forget_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_forget)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_forget = Mock()
         monkeypatch.setattr(doit_cmd, "doit_forget", mock_forget)
         doit_cmd.cmd_main(["forget",])
 
-        expected = [('dependencyFile', '.doit.db'),
+        expected = [('dependency_file', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
                     ('outstream', sys.stdout),
-                    ('forgetTasks', mock_get_tasks()['config']['default_tasks']),
+                    ('forget_tasks', mock_get_tasks()['config']['default_tasks']),
                     ]
 
         assert len(expected) == len(argspec[0])
@@ -168,15 +169,15 @@ class TestInterface(object):
 
     def test_doit_ignore_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_ignore)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_ignore = Mock()
         monkeypatch.setattr(doit_cmd, "doit_ignore", mock_ignore)
         doit_cmd.cmd_main(["ignore", "b"])
 
-        expected = [('dependencyFile', '.doit.db'),
+        expected = [('dependency_file', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
                     ('outstream', sys.stdout),
-                    ('ignoreTasks', ['b']),
+                    ('ignore_tasks', ['b']),
                     ]
 
         assert len(expected) == len(argspec[0])
@@ -187,7 +188,7 @@ class TestInterface(object):
 
     def test_doit_auto_args(self, monkeypatch):
         argspec = inspect.getargspec(doit_cmd.doit_auto)
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_auto = Mock()
         monkeypatch.setattr(doit_cmd, "doit_auto", mock_auto)
         doit_cmd.cmd_main(["auto", "b"])
@@ -207,7 +208,7 @@ class TestInterface(object):
 
 class TestErrors(object):
     def test_interrupt(self, monkeypatch):
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         def my_raise(*args):
             raise KeyboardInterrupt()
         mock_cmd = Mock(side_effect=my_raise)
@@ -215,18 +216,21 @@ class TestErrors(object):
         py.test.raises(KeyboardInterrupt, doit_cmd.cmd_main, [])
 
     def test_user_error(self, capsys, monkeypatch):
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
-        mock_cmd = Mock(side_effect=main.InvalidCommand)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
+        mock_cmd = Mock(side_effect=InvalidCommand)
         monkeypatch.setattr(doit_cmd, "doit_run", mock_cmd)
         got = doit_cmd.cmd_main([])
         assert 1 == got
         out, err = capsys.readouterr()
-        assert "ERROR" in out
+        assert "ERROR" in err
 
     def test_internal_error(self, capsys, monkeypatch):
-        monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
+        monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_cmd = Mock(side_effect=Exception)
         monkeypatch.setattr(doit_cmd, "doit_run", mock_cmd)
         got = doit_cmd.cmd_main([])
         assert 1 == got
-        # FIXME I cant test output because sys.__stderr__ is used...
+        out, err = capsys.readouterr()
+        # traceback from Exception (this case code from mock lib)
+        assert "raise self.side_effect" in err
+
