@@ -19,32 +19,28 @@ PROGRAM = "python %s/sample_process.py" % TEST_PATH
 class TestTaskCheckInput(object):
 
     def testOkType(self):
-        task.Task.check_attr_input('xxx', 'attr', [], [int, list])
+        task.Task.check_attr_input('xxx', 'attr', [], ([int, list],[]))
 
     def testOkValue(self):
-        task.Task.check_attr_input('xxx', 'attr', None, [list, None])
+        task.Task.check_attr_input('xxx', 'attr', None, ([list], [None]))
 
     def testFailType(self):
         py.test.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
-                      'attr', int, [list, False])
+                      'attr', int, ([list], [False]))
 
     def testFailValue(self):
         py.test.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
-                      'attr', True, [list, False])
+                      'attr', True, ([list], [False]))
 
 
 
-class TestTask(object):
+class TestTaskInit(object):
 
     def test_groupTask(self):
         # group tasks have no action
         t = task.Task("taskX", None)
         assert t.actions == []
 
-    def test_repr(self):
-        t = task.Task("taskX",None,('t1','t2'))
-        assert "<Task: taskX>" == repr(t), repr(t)
-
     def test_dependencySequenceIsValid(self):
         task.Task("Task X", ["taskcmd"], file_dep=["123","456"])
 
@@ -55,45 +51,6 @@ class TestTask(object):
         py.test.raises(task.InvalidTask, task.Task,
                       "Task X",["taskcmd"], file_dep=filePath)
 
-    # dependency types going to the write place
-    def test_dependencyTypes(self):
-        dep = ["file1.txt",":taskX","file2", "?res1", ":stu*"]
-        task_dict = {'name': "MyName",
-                     'actions': ["MyAction"],
-                     'dependencies': dep}
-        t = task.dict_to_task(task_dict)
-        assert t.task_dep == [dep[1][1:], dep[3][1:]]
-        assert t.file_dep == [dep[0],dep[2]]
-        assert t.result_dep == [dep[3][1:]]
-        assert t.wild_dep == [dep[4][1:]]
-
-    def test_dependencyTrueRunonce(self):
-        t = task.Task("Task X",["taskcmd"], file_dep=[True])
-        assert t.run_once
-
-    def test_dependencyFalseRunalways(self):
-        t = task.Task("Task X",["taskcmd"], file_dep=[False])
-        assert t.run_always
-
-    def test_dependencyNoneIgnored(self):
-        t = task.Task("Task X",["taskcmd"], file_dep=[None])
-        assert not t.run_once
-        assert not t.run_always
-
-    def test_runOnce_or_fileDependency(self):
-        py.test.raises(task.InvalidTask, task.Task,
-                      "Task X",["taskcmd"], file_dep=[True,"whatever"])
-
-
-    def test_title(self):
-        t = task.Task("MyName",["MyAction"])
-        assert "MyName" == t.title()
-
-
-    def test_custom_title(self):
-        t = task.Task("MyName",["MyAction"], title=(lambda x: "X%sX" % x.name))
-        assert "X%sX"%str(t.name) == t.title(), t.title()
-
     def test_options(self):
         # when task is created, options contain the default values
         p1 = {'name':'p1', 'default':'p1-default'}
@@ -110,6 +67,77 @@ class TestTask(object):
         t = task.Task("t", ["q"])
         assert t.run_status is None
 
+    def test_dependencyTrueRunOnce(self):
+        t = task.Task("Task X",["taskcmd"], run_once=True)
+        assert t.run_once
+
+
+class TestTaskUpToDate(object):
+
+    def test_dependencyFalseRunalways(self):
+        t = task.Task("Task X",["taskcmd"], uptodate=[False])
+        assert t.uptodate == [False]
+
+    def test_dependencyNoneIgnored(self):
+        t = task.Task("Task X",["taskcmd"], uptodate=[None])
+        assert not t.run_once
+        assert t.uptodate == [None]
+
+    # deprecate bool and none values for file_dep. to be removed on 0.11
+    def test_deprecation(self):
+        t = task.Task("Task x", ["taskcmd"], file_dep=[True, False, None])
+        assert 0 == len(t.file_dep)
+        assert True == t.run_once
+        assert [False, None] == t.uptodate
+
+class TestTaskExpandFileDep(object):
+
+    def test_dependencyStringIsFile(self):
+        my_task = task.Task("Task X", ["taskcmd"], file_dep=["123","456"])
+        assert set(["123","456"]) == my_task.file_dep
+
+    def test_runOnce_or_fileDependency(self):
+        py.test.raises(task.InvalidTask, task.Task, "Task X",
+                       ["taskcmd"], file_dep=["whatever"], run_once=True)
+
+    def test_file_dep_must_be_string(self):
+        py.test.raises(task.InvalidTask, task.Task, "Task X", ["taskcmd"],
+                       file_dep=[['aaaa']])
+
+
+class TestTaskDeps(object):
+
+    def test_task_dep(self):
+        my_task = task.Task("Task X", ["taskcmd"], task_dep=["123","4*56"])
+        assert ["123"] == my_task.task_dep
+        assert ["4*56"] == my_task.wild_dep
+
+    def test_expand_result(self):
+        my_task = task.Task("Task X", ["taskcmd"], result_dep=["123"])
+        assert ["123"] == my_task.result_dep
+        assert ["123"] == my_task.task_dep
+
+    def test_calc_dep(self):
+        my_task = task.Task("Task X", ["taskcmd"], calc_dep=["123"])
+        assert ["123"] == my_task.calc_dep
+        assert ["123"] == my_task.calc_dep_stack
+
+    def test_update_deps(self):
+        my_task = task.Task("Task X", ["taskcmd"], file_dep=["fileX"],
+                            calc_dep=["calcX"], result_dep=["resultX"],
+                            uptodate=[None])
+        my_task.update_deps({'file_dep': ['fileY'],
+                             'task_dep': ['taskY'],
+                             'calc_dep': ['calcX', 'calcY'],
+                             'result_dep': ['resultY'],
+                             'uptodate': [True],
+                             })
+        assert set(['fileX', 'fileY']) == my_task.file_dep
+        assert ['resultX', 'taskY', 'resultY'] == my_task.task_dep
+        assert ['calcX', 'calcY'] == my_task.calc_dep
+        assert ['resultX', 'resultY'] == my_task.result_dep
+        assert [None, True] == my_task.uptodate
+
 
 class TestTask_Getargs(object):
     def test_ok(self):
@@ -124,6 +152,26 @@ class TestTask_Getargs(object):
                               't3', None, getargs=getargs)
 
 
+class TestTaskTitle(object):
+
+    def test_title(self):
+        t = task.Task("MyName",["MyAction"])
+        assert "MyName" == t.title()
+
+
+    def test_custom_title(self):
+        t = task.Task("MyName",["MyAction"], title=(lambda x: "X%sX" % x.name))
+        assert "X%sX"%str(t.name) == t.title(), t.title()
+
+
+
+class TestTaskRepr(object):
+
+    def test_repr(self):
+        t = task.Task("taskX",None,('t1','t2'))
+        assert "<Task: taskX>" == repr(t), repr(t)
+
+
 class TestTaskActions(object):
     def test_success(self):
         t = task.Task("taskX", [PROGRAM])
@@ -348,121 +396,3 @@ class TestDictToTask(object):
     def testDictMissingFieldAction(self):
         py.test.raises(action.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
 
-
-class TestCmdExpandAction(object):
-
-    def test_task_meta_reference(self):
-        cmd = "python %s/myecho.py" % TEST_PATH
-        cmd += " %(dependencies)s - %(changed)s - %(targets)s"
-        dependencies = ["data/dependency1", "data/dependency2", ":dep_on_task"]
-        targets = ["data/target", "data/targetXXX"]
-        t = task.Task('formating', [cmd], dependencies, targets)
-        t.dep_changed = ["data/dependency1"]
-        t.execute()
-
-        got = t.actions[0].out.split('-')
-        assert t.file_dep == got[0].split(), got[0]
-        assert t.dep_changed == got[1].split(), got[1]
-        assert targets == got[2].split(), got[2]
-
-    def test_task_options(self):
-        cmd = "python %s/myecho.py" % TEST_PATH
-        cmd += " %(opt1)s - %(opt2)s"
-        t = task.Task('with_options', [cmd])
-        t.options = {'opt1':'3', 'opt2':'abc def'}
-        t.execute()
-        got = t.actions[0].out.strip()
-        assert "3 - abc def" == got, repr(got)
-
-
-
-class TestPythonActionPrepareKwargsMeta(object):
-
-    def pytest_funcarg__task_depchanged(self, request):
-        def create_task_depchanged():
-            t_depchanged = task.Task('name',None,['dependencies'],['targets'])
-            t_depchanged.dep_changed = ['changed']
-            return t_depchanged
-        return request.cached_setup(
-            setup=create_task_depchanged,
-            scope="function")
-
-    def test_no_extra_args(self, task_depchanged):
-        def py_callable():
-            return True
-        my_action = action.PythonAction(py_callable)
-        my_action.task = task_depchanged
-        my_action.execute()
-
-    def test_keyword_extra_args(self, task_depchanged):
-        got = []
-        def py_callable(arg=None, **kwargs):
-            got.append(kwargs['targets'])
-            got.append(kwargs['dependencies'])
-            got.append(kwargs['changed'])
-        my_action = action.PythonAction(py_callable)
-        my_action.task = task_depchanged
-        my_action.execute()
-        assert got == [['targets'], ['dependencies'], ['changed']], got
-
-    def test_named_extra_args(self, task_depchanged):
-        got = []
-        def py_callable(targets, dependencies, changed):
-            got.append(targets)
-            got.append(dependencies)
-            got.append(changed)
-        my_action = action.PythonAction(py_callable)
-        my_action.task = task_depchanged
-        my_action.execute()
-        assert got == [['targets'], ['dependencies'], ['changed']]
-
-    def test_mixed_args(self, task_depchanged):
-        got = []
-        def py_callable(a, b, changed):
-            got.append(a)
-            got.append(b)
-            got.append(changed)
-        my_action = action.PythonAction(py_callable, ('a', 'b'))
-        my_action.task = task_depchanged
-        my_action.execute()
-        assert got == ['a', 'b', ['changed']]
-
-    def test_extra_arg_overwritten(self, task_depchanged):
-        got = []
-        def py_callable(a, b, changed):
-            got.append(a)
-            got.append(b)
-            got.append(changed)
-        my_action = action.PythonAction(py_callable, ('a', 'b', 'c'))
-        my_action.task = task_depchanged
-        my_action.execute()
-        assert got == ['a', 'b', 'c']
-
-    def test_extra_kwarg_overwritten(self, task_depchanged):
-        got = []
-        def py_callable(a, b, **kwargs):
-            got.append(a)
-            got.append(b)
-            got.append(kwargs['changed'])
-        my_action = action.PythonAction(py_callable, ('a', 'b'), {'changed': 'c'})
-        my_action.task = task_depchanged
-        my_action.execute()
-        assert got == ['a', 'b', 'c']
-
-    def test_extra_arg_default_disallowed(self, task_depchanged):
-        def py_callable(a, b, changed=None): pass
-        my_action = action.PythonAction(py_callable, ('a', 'b'))
-        my_action.task = task_depchanged
-        py.test.raises(task.InvalidTask, my_action.execute)
-
-class TestPythonActionOptions(object):
-    def test_task_options(self):
-        got = []
-        def py_callable(opt1, opt3):
-            got.append(opt1)
-            got.append(opt3)
-        t = task.Task('with_options', [py_callable])
-        t.options = {'opt1':'1', 'opt2':'abc def', 'opt3':3}
-        t.execute()
-        assert ['1',3] == got, repr(got)
-
